package cn.iocoder.yudao.module.order.service.order;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderConsignor.OrderConsignorDO;
import cn.iocoder.yudao.module.order.vo.orderConsignor.OrderConsignorCreateReqVO;
import cn.iocoder.yudao.module.order.vo.orderConsignor.OrderConsignorQueryVO;
import cn.iocoder.yudao.module.order.vo.orderConsignor.OrderConsignorUpdateReqVO;
import org.springframework.scheduling.annotation.Async;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 订单发货人信息 Service 接口
 *
 * @author 系统管理员
 */
public interface OrderConsignorService extends IService<OrderConsignorDO> {

    /**
     * 创建订单发货人信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOrderConsignor(@Valid OrderConsignorCreateReqVO createReqVO);

    /**
     * 更新订单发货人信息
     *
     * @param updateReqVO 更新信息
     */
    void updateOrderConsignor(@Valid OrderConsignorUpdateReqVO updateReqVO);

    /**
     * 删除订单发货人信息
     *
     * @param id 编号
     */
    void deleteOrderConsignor(Long id);

    /**
     * 获得订单发货人信息
     *
     * @param id 编号
     * @return 订单发货人信息
     */
    OrderConsignorDO getOrderConsignor(Long id);

    /**
     * 获得订单发货人信息
     *
     * @param orderId 编号
     * @return 订单发货人信息
     */
    OrderConsignorDO getOrderConsignorByOrderId(Long orderId);

    /**
     * 获得订单发货人信息列表
     *
     * @param ids 编号
     * @return 订单发货人信息列表
     */
    List<OrderConsignorDO> getOrderConsignorList(Collection<Long> ids);

    /**
     * 获得订单发货人信息分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 订单发货人信息分页
     */
    PageResult<OrderConsignorDO> getOrderConsignorPage(OrderConsignorQueryVO query, PageVO page);

    /**
     * 获得订单发货人信息列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 订单发货人信息列表
     */
    List<OrderConsignorDO> getOrderConsignorList(OrderConsignorQueryVO query);


}
